use crate::controllers::{HttpResult, LibVirtReq};
use crate::libvirt_lib_structures::XMLUuid;
use crate::libvirt_rest_structures::NetworkInfo;
use actix_web::{web, HttpResponse};

#[derive(serde::Serialize, serde::Deserialize)]
pub struct NetworkID {
    uid: XMLUuid,
}

/// Create a new network
pub async fn create(client: LibVirtReq, req: web::Json<NetworkInfo>) -> HttpResult {
    let network = match req.0.to_virt_network() {
        Ok(d) => d,
        Err(e) => {
            log::error!("Failed to extract network info! {e}");
            return Ok(HttpResponse::BadRequest().body(e.to_string()));
        }
    };
    let uid = client.update_network(network).await?;

    Ok(HttpResponse::Ok().json(NetworkID { uid }))
}

/// Get the list of networks
pub async fn list(client: LibVirtReq) -> HttpResult {
    let networks = client
        .get_full_networks_list()
        .await?
        .into_iter()
        .map(|n| NetworkInfo::from_xml(n).unwrap())
        .collect::<Vec<_>>();

    Ok(HttpResponse::Ok().json(networks))
}

/// Get the information about a single network
pub async fn get_single(client: LibVirtReq, req: web::Path<NetworkID>) -> HttpResult {
    let network = NetworkInfo::from_xml(client.get_single_network(req.uid).await?)?;

    Ok(HttpResponse::Ok().json(network))
}

/// Update the information about a single network
pub async fn update(
    client: LibVirtReq,
    path: web::Path<NetworkID>,
    body: web::Json<NetworkInfo>,
) -> HttpResult {
    let mut network = body.0.to_virt_network()?;
    network.uuid = Some(path.uid);
    client.update_network(network).await?;

    Ok(HttpResponse::Ok().json("Network updated"))
}

/// Delete a network
pub async fn delete(client: LibVirtReq, path: web::Path<NetworkID>) -> HttpResult {
    client.delete_network(path.uid).await?;

    Ok(HttpResponse::Ok().json("Network deleted"))
}
